#!/bin/bash
#
# usage:
#   
#   sendnotification.sh -f <PbR file> -m <dcmmod result file>
#

. $HOME/etc/setenv.sh


# if sendnotification.sh is called directly:
# run as a task instead
if [ `basename $0` = "sendnotification.sh" ] ; then
	$TASKD_HOME/runshellcmd.sh -R -r 0 $0 "$@"
	exit $?
fi


LOGGER="/usr/bin/logger"
TAG="sendnotification[$$]"

FILE=""
MMOD_RESULT_FILE=""

while getopts 'f:m:' OPT; do
	case $OPT in
		f) FILE="$OPTARG" ;;
		m) MMOD_RESULT_FILE="$OPTARG" ;;
	esac
done

echo $FILE
echo $MMOD_RESULT_FILE

run_() # override run_ method
{
	PEER_CFG="$DICOM_VAR/resultsmgmt_peer.cfg"

	[ -z "$FILE" -o -z "$MMOD_RESULT_FILE" ] && return 125
	[ ! -f "$PEER_CFG" ] && return 0
	[ ! -f "$MMOD_RESULT_FILE" ] && return 0

	. $PEER_CFG
	PEER_AE_TITLE="$AE_TITLE"
	PEER_PORT="$PORT"
	PEER_SERVER="$PEER"
	PEER_PROTOCOL="$PROTOCOL"

	[ -z "$PEER_PORT" -o -z "$PEER_SERVER" -o -z "$PEER_AE_TITLE" ] && return 125

	[ -f "$DICOM_VAR/pb-scp.cfg" ] && . $DICOM_VAR/pb-scp.cfg
	OUR_AE_TITLE="$AE_TITLE"
	[ -z "$OUR_AE_TITLE" ] && OUR_AE_TITLE="$DICOM_AE_TITLE"

	TLS_OPT=""
	[ "$PEER_PROTOCOL" = "TLS" ] && TLS_OPT="+tls $SSL_SITE_KEY $SSL_SITE_CERT -ic +ps"


	$DICOM_HOME/bin/getchanges.awk $MMOD_RESULT_FILE | \
	while read sopclass uid event ; do
		scl=""
		[ "$sopclass" = "results" ] && scl="--dresults"
		[ "$sopclass" = "interpretation" ] && scl="--dinterpretation"

		echo "-- $sopclass $uid $event --"

		[ -z "$scl" -o -z "$uid" -o -z "$event" ] && return 125;

		if [ "$PEER_PROTOCOL" = "TLS" ] ; then
			$OPENSSL_PASS
		fi | \

		$DICOM_HOME/bin/eventreportscp $TLS_OPT $scl --call $PEER_AE_TITLE --aetitle $OUR_AE_TITLE \
				--event "$event" --sopiuid "$uid" --file $FILE $PEER_SERVER $PEER_PORT

		[ $? -ne 0 ] && return 1
	done

	return $?
}


print_()
{
	echo "send DICOM Results Mgmt notification [PbR file: $FILE; Mod file: $MMOD_RESULT_FILE]"
}


get_watson_message_()
{
  [ -z "$DR_WATSON_ID" ] && DR_WATSON_ID="notify.$PEER_SERVER.$PEER_PORT.$PEER_AE_TITLE"
  [ -z "$DR_WATSON_MESSAGE" ] && DR_WATSON_MESSAGE="Notifying $PEER_SERVER:$PEER_PORT:$PEER_AE_TITLE"

  if [ "$RUN_RESULT" -gt 124 ] ; then
    WATSON_MESSAGE_ID="$DR_WATSON_ID.failed"
    WATSON_MESSAGE_TEXT="FAILED: $DR_WATSON_MESSAGE"
    return
  fi

  if [ "$RUN_RESULT" -ne 0 -a "$RETRY_NUM" -gt 0 -a $(( $RETRY_NUM % 10)) -eq 0 ] ; then
    WATSON_MESSAGE_TEXT="ERROR: $DR_WATSON_MESSAGE [Retry #$RETRY_NUM]"
    WATSON_MESSAGE_ID="$DR_WATSON_ID.$RETRY_NUM"
  fi

} # get_watson_message_()

