#!/bin/gawk -f

#
# Component: dicom
#
# Description:
#    Creates HostTable for qr.cfg from MySQL query output.
#    used by ctrl
#
#
# Expected parameters
#   FS = '\t'
#   ip     	= $SERVERIP
#   port   	= $PORT
#   dae    	= $DICOM_AE_TITLE
#   ae     	= $AE_TITLE
#
# Expected fields
#
# $1 - ID
# $2 - HOSTNAME
# $3 - AE
# $4 - PORT
# $5 - PROTOCOL
# $6 - SERVICE
#
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

function service2Num(s) {
	a = serviceNum[s];
	return a == "" ? 0 : a;
}

function printEntry(pid, pae, phost, pport, pprotocol, precv, psend, pconn, pservice, em, ci, ts) {
	printf "%s = (%s, %s, %d, %s, %d, %d, %d, %08x, %s, %s, %s )\n", pid, pae, phost, pport, pprotocol, precv, psend, pconn, pservice, em, ci, ts;
}

#Collect settings for a specified device stored in setting.cfg
#
#param p_hid : the id of the device which settings have to collect
#
#meaning of prefixes: 
#	em	Empty Matching
#	ts	Transfer Syntax
#	ci	Case InSensitive
#
#	g	Global
#
#For example gem variable stores the global settings for empty matching property
#	     ts variable stores the local settings for Transfer Stntax
#
#Function modify global variables: em, sf, ts ci
function collectSettingsForDevice(p_hid) {
	tmpLine=$0
	ERRNO=""
	FS="=" 	
	inputFile= dicom_var "/devices/" p_hid ".cfg"

	em = gEmptyMatching
	sf = gStorescuFlag
	ts = gTransferSyntax
	ci = gCaseInsensitive
	st = gSendTimeout
	rt = gReceiveTimeout
	ct = gConnectionTimeout

	while ( (getline < inputFile) > 0 ) {
		if ($1 == "EMPTY_MATCHING")
			em=gensub(/"/,"","g",$2)
		if ($1 == "STORESCU_FLAG")
			sf=gensub(/"/,"","g",$2)
		if ($1 == "TRANSFER_SYNTAX")
			ts=gensub(/"/,"","g",$2)
		if ($1 == "CASE_INSENSITIVE")
			ci=gensub(/"/,"","g",$2)
		if ($1 ~ /SEND_TIMEOUT/)
			st=gensub(/"/,"","g",$2)
		if ($1 ~ /RECEIVE_TIMEOUT/)
			rt=gensub(/"/,"","g",$2)
		if ($1 ~ /CONNECTION_TIMEOUT/)
			ct=gensub(/"/,"","g",$2)
	}
	if ( ERRNO != "" ) {
		print "Error reading '" inputFile "': " ERRNO > "/dev/stderr"
	}
	close(inputFile)
	
	FS="\t"
	$0=tmpLine
}


BEGIN {
	# Store SCP
	serviceNum["StrP"] = 0;
	# Store SCU
	serviceNum["StrU"] = 1;
	# Query Retrieve SCP
	serviceNum["QR_P"] = 0;
	# Query Retrieve SCU
	serviceNum["QR_U"] = 2;
	# Modality Worklist SCP
	serviceNum["MWLP"] = 0;
	# Modality Worklist SCU
	serviceNum["MWLU"] = 4;
	# Results Management SCP
	serviceNum["RM_P"] = 16;
	# Results Management SCU
	serviceNum["RM_U"] = 32;
	# Storage Commitment SCU
	serviceNum["ComU"] = 64;
	# Storage Commitment SCP
	serviceNum["ComP"] = 128;
	# Modality Performed Procedure Step SCU
	serviceNum["MPPS"] = 256;

	allservices = 512; # Modality Performed Procedure Step SCP
	for (i in serviceNum) {
		allservices += serviceNum[i];
	}

	idl= "";
	print "HostTable BEGIN";

	hid="";
	hhost="";
	hae="";
	hport="";
	hprotocol="";
	hrecv="";
	hsend="";
	hservice = 0;
	
	# Factory default settings
	gEmptyMatching = "false";
	gCaseInsensitive = "true";
	gStorescuFlag = "-R";
	gSendTimeout = 30;
	gReceiveTimeout = 30;
	gConnectTimeout = 30;
	gTransferSyntax = "xe";

	
	FS = "=" 	
	dicom_var = ENVIRON["DICOM_VAR"]
	inputFile = dicom_var "/pb-scp.cfg"
	while (getline < inputFile) {
#		print "DEBUG(" inputFile "): " $1 " :: " $2;
		if ($1 ~ /EMPTY_MATCHING/)
			gEmptyMatching=gensub(/"/,"","g",$2)
		if ($1 ~ /STORESCU_FLAG/)
			gStorescuFlag=gensub(/"/,"","g",$2)
		if ($1 ~ /TRANSFER_SYNTAX/)
			gTransferSyntax=gensub(/"/,"","g",$2)
		if ($1 ~ /CASE_INSENSITIVE/)
			gCaseInsensitive=gensub(/"/,"","g",$2)
		if ($1 ~ /SEND_TIMEOUT/)
			gSendTimeout=gensub(/"/,"","g",$2)
		if ($1 ~ /RECEIVE_TIMEOUT/)
			gReceiveTimeout=gensub(/"/,"","g",$2)
		if ($1 ~ /CONNECTION_TIMEOUT/)
			gConnectionTimeout=gensub(/"/,"","g",$2)
	}
	close(inputFile)
	FS="\t"
	
}

($1 == "") {
	next;
}

($1 == hid) {
	hservice = hservice + service2Num($6);
	next;
}


{
	#	print hid
	#print $1
	if (hid != "") {
		collectSettingsForDevice(hid)
	#print "aefter " $1
		printEntry(hid, hae, hhost, hport, hprotocol, rt, st, ct, hservice, em, ci, ts);
	}

	hid = $1;
	hhost = $2;
	hae = $3;
	hport = $4;
	hprotocol = $5;
	hservice = service2Num($6);
	idl = idl hid ",";

	if (hae == "" || hae == "*") {
		hae = dae;
	}

	if (hport == "") {
		hport="104";
	}

	if (hprotocol == "S") {
		hprotocol = "TLS";
	}
	else {
		hprotocol = "TCP";
	}
}

END {
	if (hid != "") {
		collectSettingsForDevice(hid)
		printEntry(hid, hae, hhost, hport, hprotocol, rt, st, ct, hservice, em, ci, ts);
	}

	printEntry("_-_LOOPBACK_-_", ae, ip, port, "TCP", gReceiveTimeout, gSendTimeout, gConnectionTimeout, allservices, gEmptyMatching, gCaseInsensitive, gTransferSyntax);

	idl = idl "_-_LOOPBACK_-_";

	print "_-_ALLOWED_-_ = " idl;
	print "HostTable END";
}
