#pragma once
#include <string>
#include <map>
#include <cinttypes>

using namespace std;

class BlobEntry {

public:
	string fileName;
	string prefixTag;
	int64_t offset;// = 0;
	int32_t size;// = 0;
	int32_t nameLength;// = 0;

	BlobEntry( const string& fileName, int32_t nameLength, int64_t offset, int32_t size ) {
		this->nameLength = nameLength;
		size_t idxH = fileName.find( '#' );
		if ( idxH != string::npos ) {
			this->prefixTag = fileName.substr( 0, idxH );
			this->fileName = fileName.substr( idxH+1 );
		}
		else {
			this->fileName = fileName;
		}
		this->offset = offset;
		this->size = size;
	}

	int getHeaderEntrySize() {
		return 2 + nameLength + 8;
	}
};


class BlobHeader {
	public:
		const static int blobHeaderLength = 128;
		const static int32_t blobMagiX = 0x4C425245;
		const static int32_t blobHeaderVersion = 0x01;
		// unit8_t reserved[116] // for future use
};


class FileBlob {

private:
	static string DELPREFIX; //= "ERDELETE";

	string blobFileName;// = NULL;

	bool needBlobUpdate;// = false;

	/** blob map, key = name **/
	map<string, BlobEntry*> blobMap;// = new HashMap<String, BlobEntry>();

	FILE* blobRAF;// = NULL;

	int64_t predictedEnd;// = 0;
	int64_t payloadOffset;// = 4;

public:
	FileBlob(string blobFileName) {
		this->blobFileName = blobFileName;
		needBlobUpdate = false;
		blobRAF = NULL;
		predictedEnd = 0;
		payloadOffset = BlobHeader::blobHeaderLength;
	}
	virtual ~FileBlob();

	bool parseBlob();
	void printEntries();
	void extractAll();
	BlobEntry* findBlobEntry( const string& fileName );
	int getEntrySize( BlobEntry* entry ) ;
	int readEntryIntoBuffer( BlobEntry* entry, int entryOffset, uint8_t* buffer, int bufferOffset, int bytesToRead );

private:
	void closeBlobFileChannel();
	void clearEntries();
	int64_t getFileSize();

	bool readHeaderEntry();
	int32_t parseBlobChunk();
	bool putBlobEntry( BlobEntry* entry );
	bool delBlobEntry( const string& fileName );

	//void writeBlob();
	//void compactBlob();
	//static void compact( const string& blobFileName );
	//static int readEntryToBuffer( const string& blobFileName, const string& fileName, int maxSize, JBYTE* buffer );


#if 0
			/**
			 *
			 * Helper functions
			 *
			 */

			/**
			 * Converts a number to a byte array.
			 *
			 * @param n
			 *          the number to convert
			 * @param nb
			 *          the number of bytes to return
			 *
			 * @return the byte array representation of the number
			 */
			static byte[] numberToLittleEndian( long n, int nb ) {
				byte[] b = new byte[ nb ];

				for ( int i = 0; i < nb; i++ ) {
					b[ i ] = (byte) ( n & 255 );
					n = n >> 8;
				}

				return b;
			}

			/**
			 * Creates the byte array representation of a <code>short</code> value.
			 * The length of the resulting byte array will be 2.
			 *
			 * @param n
			 *          the value to convert
			 *
			 * @return the byte array created from the <code>short</code> value
			 */
			static byte[] shortToLittleEndian( short n ) {
				return numberToLittleEndian( n, 2 );
			}

			static int littleEndianToShort( byte[] buffer, int offset ) {
				return ByteBuffer.wrap( buffer, offset, 2 ).order( ByteOrder.LITTLE_ENDIAN ).getShort();
			}


			/**
			 * Creates the byte array representation of an <code>int</code> value.
			 * The length of the resulting byte array will be 4.
			 *
			 * @param n
			 *          the value to convert
			 *
			 * @return the byte array created from the <code>int</code> value
			 */
			static byte[] intToLittleEndian( int n ) {
				return numberToLittleEndian( n, 4 );
			}

			static int littleEndianToInt( byte[] buffer, int offset ) {
				return ByteBuffer.wrap( buffer, offset, 4 ).order( ByteOrder.LITTLE_ENDIAN ).getInt();
			}

			static String littleEndianToStringUTF8( byte[] buffer, int offset ) {
				int l = littleEndianToShort( buffer, offset );
				String s = new String( Arrays.copyOfRange( buffer, offset + 2, offset + 2 + l ), Charset.forName( "UTF-8" ) );
				return s;
			}


		}
#endif //0


};


