/*
 * Component: imutils
 *
 * Description:
 *     Semaphore to lock/unlock, using memory instead of files.
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef __IMUTILS_MEM_SEMAPHORE_H_INCLUDED__
#define __IMUTILS_MEM_SEMAPHORE_H_INCLUDED__


#include <sys/sem.h>

/**
 * Semaphore to lock/unlock, using memory instead of files
 */
class IMMemSemaphore {

private:
	int m_semID;
	struct sembuf m_op;
	bool m_unlock;

public:

	/**
	 * Creates a new Semaphore
	 *
	 * @param key The ID of the semaphore
	 * @param unlockAutomatic If true, there's no need to unlock the semaphore as it will unlock in the object destructor
	 * 		 beware: if set, and you unlock it manually, you will get error message in error.log in the destructor
	 *
	 */
	IMMemSemaphore( int key, bool unlockAutomatic = true );

	/**
	 *	Destructor
	 */
	~IMMemSemaphore();

	/**
	 *	Initialize the semaphore, must be called once (server start time), but only once
	 *
	 * @return if succesful
	 */
	bool init();

	/**
	 * Locks the semaphore, if cannot, waits until locking is possible
	 *
	 * @return if no error occured
	 */
	bool lock();

	/**
	 * Unlocks the semaphore
	 *
	 * @return if no error occured
	 */
	bool unlock();

	/**
	 * Clears the semaphore, must be called once (server stop time), but only once
	 *
	 * @return if succeeded
	 */
	bool close();
};

#endif // __IMUTILS_MEM_SEMAPHORE_H_INCLUDED__
