#ifndef __STATERANGESCONFIG_H__
#define __STATERANGESCONFIG_H__

#include <string>
#include <fstream>
#include <vector>

#include "imutils/IMProperties.h"


/**
 * A range element in the stateranges.xml.
 */
class StateRange {
public:
	StateRange(const std::string& type, const std::string& name, int min, int max, bool custom);
	const std::string& getType() const;
	const std::string& getName() const;
	int getMin() const;
	int getMax() const;
	bool isCustom() const;
private:
	std::string m_type;	// eprange, dicomrange, hl7range, etc...
	std::string m_name;	// the label bound to the range
	int m_min;	// lower bound of the range
	int m_max;	// higher bound of the range
	bool m_custom;	// custom or default range; custom ranges can be find in a separate xml file
};


/**
 * stateranges.xml reader class.
 */
class StaterangesConfig {
public:
	StaterangesConfig();
	~StaterangesConfig();

	/**
	 * Returns true if the xml file exists and its size is not zero.
	 */
	bool xmlFileExists(const std::string& staterangesXml);

	/**
	 * Opens and parses default and custom xml files to load available ranges.
	 * Returns true if at least the default xml could be successfully loaded.
	 */
	bool loadXml();

	/**
	 * True if the loadXml function succeeded.
	 */
	bool isLoaded() const;

	/**
	 * Returns the label belongs to the given range value needs to be shown.
	 */
	const std::string& getRangeLabel(int rangeCode);

	/**
	 * Returns the label needs to be shown if no label belongs to the range
	 */
	const std::string& getUnknownRangeLabel() const;

private:
	std::ifstream m_xmlstream;
	bool m_loaded;
	std::vector<StateRange> m_stateranges;

	bool loadXml(const std::string& staterangesXml, bool custom);
	bool parseXml(bool custom);
	void customizeLabel(const IMProperties& messages, std::string& statusname, bool custom);
};

#endif // __STATERANGESCONFIG_H__
