#ifndef UTILS_H_
#define UTILS_H_

#include <time.h>

#include <stdint.h>

#include <string>


void mynanosleep(unsigned long int ns);
void mymilisleep(unsigned long int ms);

pid_t mygettid();

int mymemcmp(const void* pOne, size_t nOne, const void* pTwo, size_t nTwo);

/**
 * Returns the current value of the high-resolution timer in nanoseconds.
 */
long long nanoTime();

double nanoToSec(long long nanoTime);

double nanoToMilisec(long long nanoTime);

template <size_t align>
char* alignTo(char* p) // power of 2
{
	return reinterpret_cast<char*>(
		(reinterpret_cast<uintptr_t>(p) + static_cast<uintptr_t>(align - 1)) & ~static_cast<uintptr_t>(align - 1)
	);
}

size_t roundUpTo(size_t n, size_t divider);

class FunctionMeasurer
{
	static size_t s_Level;

	const char* m_szFunctionName;
	long long m_StartTime;

	std::string indent() const;
public:
	FunctionMeasurer(const char* szFunctionName);
	~FunctionMeasurer();
	void measure(const char* szMessage = NULL) const;
};

#define MEASURE_FUNCTION FunctionMeasurer _fmeas(__FUNCTION__);


template <class T>
class Singleton {
protected:
	static T* s_pInstance;

public:
	static T& GetInstance() {
		if (NULL == s_pInstance) {
			s_pInstance = new T();
		}
		return *s_pInstance;
	}

	static void DestroyInstance() {
		delete s_pInstance;
		s_pInstance = NULL;
	}
};


template <class T>
T* Singleton<T>::s_pInstance = NULL;

#endif /* UTILS_H_ */
