/**
 * Component: utils
 *
 * Description:
 *     Registering/Deregistering DICOM TS codecs
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */


#ifndef CODECREGISTRY_H_
#define CODECREGISTRY_H_

#include <dcmtk/dcmdata/dcdatset.h>
#include <dcmtk/dcmdata/dcxfer.h>

/**
 * Class registering supported image compression codecs
 *
 * To register codecs, simply create an instance of this class.
 * Destructor will deregister the codecs
 */
class CodecRegistry {

	public:

		/**
		 * Constructor for creating a CodecRegistry instance.
		 * Supported image compression codecs will automatically be registered.
		 */
		CodecRegistry();

		/**
		 * Destructor for destroying this CodecRegistry object.
		 * Will deregistered image compression codecs
		 */
		~CodecRegistry();

		/**
		 * Creates representation parameter for converting the pixel data in dataset to the given transfer syntax.
		 * This method will not convert the dataset, but creating representation parameter for some codecs may depend on the content of the dataset.
		 * The created representation parameter should be freed (deleted) by the caller.
		 *
		 * @param dataset  dataset to be converted
		 * @param xfer     transfer syntax the dataset to be converted to
		 *
		 * @return representation parameter required by the codec or NULL if conversion does not require representation parameter
		 */
		static DcmRepresentationParameter * getRepresentationParameter( DcmDataset & dataset, E_TransferSyntax xfer );
};

#endif /* CODECREGISTRY_H_ */
