#!/bin/bash
#
# Connects a PbD file to an SRt file
#
# Links a PbD and an SRt file, fixes the
#  SRt.(f215,1032).(f215,1036)        [ := PbD.(f215,1033).(0008,0018)]
#  SRt.(f215,1032).(4008,0103)        [ := '<studyUID>/<SRt.(f215,1032).(f215,1036)>']
#  PbD.(f215,1033).(f215,1035)        [ := SRt.(f215,1032).(0008,0018)]
# tags in the SRt and PbD files
#
#  Last Modified by   : $Author$
#  Last modificied at : $Date$
#  RCS File Name      : $RCSfile$
#  RCS Revision       : $Revision$
#  Locked by          : $Locker$
#  Symbolic name      : $Name$
#  State              : $State$
#

. $HOME/etc/setenv.sh


DIRNAME=`dirname $0`

errorExit_()
{
	rm -f replaceUIDs.$$ filelist.$$ 2>/dev/null
	echo "$1" >&2
	exit 1
}


reportFile=""
dictationFile=""
outputDir=""
resStudy="report"

while getopts 'r:d:o:s:' OPT; do
	case $OPT in
		r) reportFile="$OPTARG" ;;
		d) dictationFile="$OPTARG" ;;
		o) outputDir="$OPTARG" ;;
		s) resStudy="$OPTARG" ;;
		*) ERRMSG="Invalid_Parameter:$OPT" ;;
	esac
done


[ -z "$reportFile" ] && errorExit_ "Missing argument: reportFile: (usage: -r <report file> -d <dictation file> -o <output dir> -s {report|dictation})"
[ -z "$dictationFile" ] && errorExit_ "Missing argument: dictationFile: (usage: -r <report file> -d <dictation file> -o <output dir> -s {report|dictation})"
[ -z "$outputDir" ] && errorExit_ "Missing argument: outputDir: (usage: -r <report file> -d <dictation file> -o <output dir> -s {report|dictation})"

[ ! -f "$reportFile" ] && errorExit_ "Report file $reportFile does not exist"
[ ! -f "$dictationFile" ] && errorExit_ "Dictation file $dictationFile does not exist"
[ ! -d "$outputDir" ] && errorExit_ "Output directory $outputDir does not exist"

[ "$resStudy" != "report" -a "$resStudy" != "dictation" ] && errorExit_ "Invalid value for option '-s': valid values: {report|dictation}"


studyUID=`$DCMTK_HOME/bin/dcmdump +P 0020,000d $reportFile | awk '{gsub("[\\]\\[]","",$3); print $3}'`
intUID=`$DICOM_HOME/bin/dcmdump +p +P 0008,0018 $reportFile |\
		awk '( $1 ~ /^\(f215,..32\)\.\(0008,0018\)/ ) {gsub("[\\]\\[]","",$3); print $3}'`

modFile=modfile.$$
dcmFile=dcmfile.$$

cat <<ENDOFMODFILE > $modFile
(f215,1041) SQ # PBCreateOnlySequence
  (fffe,e000) na # Item
    (f215,1032) SQ # PBInterpretationSequence
      (fffe,e000) na # Item
        (0008,0018) UI [$intUID] # SOPInstanceUID
        (4008,0103) LO [$studyUID/1.1] # ReferenceToRecordedSound
        (f215,1036) UI [1.1] # PBReferencedDictationUID
      (fffe,e00d) na # ItemDelimitationItem
    (fffe,e0dd) na # SequenceDelimitationItem
  (fffe,e00d) na # ItemDelimitationItem
(fffe,e0dd) na # SequenceDelimitationItem
ENDOFMODFILE

$DCMTK_HOME/bin/dump2dcm $modFile $dcmFile
RES=$?
rm -f $modFile
[ $RES -ne 0 ] && errorExit_ "Could not create dcmmod file"

TMPReportFile="$TMP/$reportFile.$$"

$DCVIEWER_HOME/dcmmod -f "$reportFile" -m "$dcmFile" -o "$TMPReportFile"
RES=$?

rm -f $dcmFile
[ $RES -ne 0 ] && errorExit_ "Could not add dictation reference to SRt file"

$DICOM_HOME/bin/dcmdump +L +p +P 0008,0018 +P f215,1036 +P f215,1035 +P 0020,000d +P 0020,000e $TMPReportFile $dictationFile | \
		$DIRNAME/createReplaceUIDs.awk -vResStudy="$resStudy" > replaceUIDs.$$
[ $? -ne 0 ] && errorExit_ "Could not create replace UID file"

ls $TMPReportFile $dictationFile > filelist.$$

$UTILS_HOME/bin/copyDicomFiles -d "$outputDir" -f filelist.$$ -l replaceUIDs.$$

[ $? -ne 0 ] && errorExit_ "Error connecting PbD to SRt"

rm -f replaceUIDs.$$ filelist.$$ $TMPReportFile 2>/dev/null

exit 0
