#!/bin/bash
#
# Component: sr
#
# Description:
#    Copies PB Figure from one SRt file to another
#
# $1 copy from SR
# $2 copy to SR
#
#  Last Modified by   : $Author$
#  Last modificied at : $Date$
#  RCS File Name      : $RCSfile$
#  RCS Revision       : $Revision$
#  Locked by          : $Locker$
#  Symbolic name      : $Name$
#  State              : $State$
#

. $HOME/etc/setenv.sh

from="$1"
to="$2"

if [ -z "$to" ] ; then
	echo "Missing argument(s): <from> <to>" >&2
	exit 1
fi

# get figure from 'from'
$DICOM_HOME/bin/dcmdump +L +P f215,1003 $from | sed 's;^.* \[\(.*\)\] .*$;\1;' > figure.$$
if [ $? -ne 0 ] ; then
	echo "Error getting PBFigure from $from" >&2
	exit 1
fi

INTSOPI=`$DICOM_HOME/bin/dcmdump +p +P 0008,0018 $to | grep f215,..32 | sed 's;^.* \[\(.*\)\] .*$;\1;'`
if [ $? -ne 0 -o -z "$INTSOPI" ] ; then
	echo "Error getting Interpretation SOP Instance UID from $to" >&2
	exit 1
fi

cat <<EOF > mod.dmp.$$
(f215,1043) SQ # PBOverWriteSequence 
  (fffe,e000) na # Item
    (f215,1032) SQ # PBInterpretationSequence
      (fffe,e000) na # Item
        (0008,0018) UI [$INTSOPI] # SOPInstanceUID
        (f215,1003) LT [FILE:figure.$$] # PBFigure
      (fffe,e00d) na # ItemDelimitationItem
    (fffe,e0dd) na # SequenceDelimitationItem
  (fffe,e00d) na # ItemDelimitationItem
(fffe,e0dd) na # SequenceDelimitationItem
EOF

$DCMTK_HOME/bin/dump2dcm mod.dmp.$$ mod.dcm.$$
if [ $? -ne 0 ] ; then
	echo "Error creating mod file" >&2
	exit 1
fi

$DCVIEWER_HOME/dcmmod -f $to -m mod.dcm.$$ -o out.$$
if [ $? -ne 0 ] ;then
	echo "Error editing $to" >&2
	exit 1
fi

echo "Output file is out.$$"

