#!/bin/gawk -f
#
# Component: sr
#
# Description:
#   Create replace UIDs file for associating a PbD file to an SRt file.
#
#  Used by connectPbD2SR.sh
#
# -vResStudy : which will be the resulted study:
#     report (default)
#     dictation
#
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

BEGIN {
	dictStudyUID = "";
	reportStudyUID = "";
	dictInstanceUID = "";
	reportInstanceUID = "";
	dictSeriesUID = "";
	reportSeriesUID = "";
	if (ResStudy != "dictation")
		ResStudy = "report"
}

{
	gsub("[][]","");
}

( $1 ~ /\(f215,..33\)\.\(0008,0018\)/ ) {
	goodDictUID = $3;
	# Dictation SOP I UID: keep the original
	print "Replace " goodDictUID " by " goodDictUID;
	next;
}

( $1 ~ /\(f215,..32\)\.\(f215,1036\)/ ) {
	badDictUID = $3;
	next;
}

( $1 ~ /\(f215,..32\)\.\(0008,0018\)/ ) {
	# Interpretation SOP I UID: keep the original
	goodIntUID = $3;
	print "Replace " goodIntUID " by " goodIntUID;
	next;
}

( $1 ~ /\(f215,..33\)\.\(f215,1035\)/ ) {
	badIntUID = $3;
	next;
}

( $1 ~ /\(f215,..31\)\.\(0008,0018\)/ ) {
	# Impression SOP I UID: keep the original
	print "Replace " $3 " by " $3;
}

($1 == "(0020,000d)") {
	# Study Instance UID - first occurance is the SRt, the second one is the PbD
	if (reportStudyUID == "") {
		reportStudyUID = $3;
		if (reportSeriesUID == "") {
			# It may happen that the series UID is missing from the SRt
			reportSeriesUID = "aa";
		}
	}
	else if (dictStudyUID == "") {
		dictStudyUID = $3;
	}
	next;
}

($1 == "(0020,000e)") {
	if (reportSeriesUID == "" ) {
		reportSeriesUID = $3;
		if (ResStudy == "report")
			print "Replace " reportSeriesUID " by " reportSeriesUID;
	}
	else if (dictSeriesUID == "") {
		dictSeriesUID = $3;
		if (ResStudy == "dictation")
			print "Replace " dictSeriesUID " by " dictSeriesUID;
	}
	next;
}
	
($1 == "(0008,0018)") {
	if (reportInstanceUID == "") {
		reportInstanceUID = $3;
		if (ResStudy == "report")
			print "Replace " reportInstanceUID " by " reportInstanceUID;
	}
	else if (dictInstanceUID == "") {
		dictInstanceUID = $3;
		if (ResStudy == "dictation")
			print "Replace " dictInstanceUID " by " dictInstanceUID;
	}
	next;
}

END {
	if (goodDictUID != "" && badDictUID != "")
		print "Replace " badDictUID " by " goodDictUID;
	if (goodIntUID != "" && badIntUID != "")
		print "Replace " badIntUID " by " goodIntUID;
	if (ResStudy == "dictation") {
		print "Replace " reportStudyUID " by " dictStudyUID;
		print "Replace " dictStudyUID " by " dictStudyUID;
	}
	if (ResStudy == "report") {
		print "Replace " reportStudyUID " by " reportStudyUID;
		print "Replace " dictStudyUID " by " reportStudyUID;
	}

}

