#!/bin/bash
#
# Removes dictation from a PbD file
#
#    $1  - PbD file
#    $2  - dictation UID
#   [$3] - output file
#
#  Last Modified by   : $Author$
#  Last modificied at : $Date$
#  RCS File Name      : $RCSfile$
#  RCS Revision       : $Revision$
#  Locked by          : $Locker$
#  Symbolic name      : $Name$
#  State              : $State$
#

. $HOME/etc/setenv.sh

errorExit_()
{
	echo "$1" >&2
	exit 1
}

[ -z "$2" ] && errorExit_ "Missing arguments:   <PbD file> <dictation UID> [<output file>]"

pbdFile="$1"
dictUID="$2"
outFile="$3"

[ ! -f "$pbdFile" ] && errorExit_ "PbD file $pbdFile does not exist"

modFile=modfile.$$
dcmFile=dcmfile.$$

cat <<ENDOFMODFILE > $modFile
(f215,1044) SQ # PBRemoveSequence
  (fffe,e000) na # Item
    (f215,1033) SQ # PBDictationSequence
      (fffe,e000) na # Item
        (0008,0018) UI [$dictUID] # SOPInstanceUID
      (fffe,e00d) na # ItemDelimitationItem
    (fffe,e0dd) na # SequenceDelimitationItem
  (fffe,e00d) na # ItemDelimitationItem
(fffe,e0dd) na # SequenceDelimitationItem
ENDOFMODFILE


$DCMTK_HOME/bin/dump2dcm $modFile $dcmFile
RES=$?
rm -f $modFile
[ $RES -ne 0 ] && errorExit_ "Could not create dcmmod file"

if [ -n "$outFile" ] ; then
	$DCVIEWER_HOME/dcmmod -f "$pbdFile" -m "$dcmFile" -o "$outFile"
	RES=$?
else
	$DCVIEWER_HOME/dcmmod -f "$pbdFile" -m "$dcmFile"
	RES=$?
fi

rm -f $dcmFile
[ $RES -ne 0 ] && errorExit_ "Could not remove dictation from PbD file"

exit 0
