#!/bin/bash
#
# Removes dictation reference from a SRt file
#
#    $1  - SRt file
#    $2  - interpretation UID
#   [$3] - output file
#
#  Last Modified by   : $Author$
#  Last modificied at : $Date$
#  RCS File Name      : $RCSfile$
#  RCS Revision       : $Revision$
#  Locked by          : $Locker$
#  Symbolic name      : $Name$
#  State              : $State$
#

. $HOME/etc/setenv.sh

errorExit_()
{
	echo "$1" >&2
	exit 1
}

[ -z "$2" ] && errorExit_ "Missing arguments:   <SRt file> <interpretation UID> [<output file>]"

srtFile="$1"
intUID="$2"
outFile="$3"

[ ! -f "$srtFile" ] && errorExit_ "SRt file $srtFile does not exist"

modFile=modfile.$$
dcmFile=dcmfile.$$

cat <<ENDOFMODFILE > $modFile
(f215,1044) SQ # PBRemoveSequence
  (fffe,e000) na # Item
    (f215,1032) SQ # PBInterpretationSequence
      (fffe,e000) na # Item
        (0008,0018) UI [$intUID] # SOPInstanceUID
        (4008,0103) LO [] # ReferenceToRecordedSound
        (f215,1036) UI [] # PBReferencedDictationUID
      (fffe,e00d) na # ItemDelimitationItem
    (fffe,e0dd) na # SequenceDelimitationItem
  (fffe,e00d) na # ItemDelimitationItem
(fffe,e0dd) na # SequenceDelimitationItem
ENDOFMODFILE


$DCMTK_HOME/bin/dump2dcm $modFile $dcmFile
RES=$?
rm -f $modFile
[ $RES -ne 0 ] && errorExit_ "Could not create dcmmod file"

if [ -n "$outFile" ] ; then
	$DCVIEWER_HOME/dcmmod -f "$srtFile" -m "$dcmFile" -o "$outFile"
	RES=$?
else
	$DCVIEWER_HOME/dcmmod -f "$srtFile" -m "$dcmFile"
	RES=$?
fi

rm -f $dcmFile
[ $RES -ne 0 ] && errorExit_ "Could not remove dictation reference from SRt file"

exit 0
