#!/bin/bash
#
#  autoget_n.sh
#
#  This script will keep retrieving studies from a PACS that doesn't support
# autoforward, if the PACS supports
#
#  - Q/R
#  - supports the optional field of (0020,1208) (Number of study related images)
#
# Usage:
#  -i IP
#  -p PORT
#  -a AETITLE
#  -r Q/R model (root, P/S/O, default 'P')
#  -c local AETITLE (default is as the server settings)
#  -b backcheck (Check only studies that are newer than this many *hours* def. 72)
#  -l locknum (send drwatson message if autoget_n.sh could not started the last locknum times due to a lock file exists; def. 5)
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh


HRS=72
QRIP=127.0.0.1
QRROOT=S
QRAE_TITLE=PBAE
LOCKNUM=5

while getopts "i:p:a:r:c:b:" OPT ; do
	case $OPT in
		i) QRIP="$OPTARG" ;;
		p) QRPORT="$OPTARG" ;;
		a) QRAE_TITLE="$OPTARG" ;;
		r) QRROOT="$OPTARG" ;;
		c) AE_TITLE="$OPTARG" ;;
		b) HRS="$OPTARG" ;;
		l) LOCKNUM=$(( $OPTARG + 0 )) ;;
	esac
done


#lock, one instance only

if [ -f /tmp/autoretrieve.lock ]; then
	STUCK=`ps auxwww | grep dcmtk/bin/move[s]cu | awk '{ a = $10; split(a,x,":"); if (x[1]>=2) print $2}'`
	if [ -n "$STUCK" ]; then
		echo killing stuck movescu at $STUCK
		kill $STUCK
	fi

	echo "$$" >> /tmp/autoretrieve.lockfailed
	FAILNUM=`cat /tmp/autoretrieve.lockfailed 2>/dev/null | wc -l`
	if [ "$LOCKNUM" -gt 0 -a "$FAILNUM" -gt $LOCKNUM ] ; then
		[ $(( $FAILNUM % $LOCKNUM )) -eq 0 ] && $DRWATSON_HOME/addmessage.sh -t ERROR -l NONE "Could not start autoget_n.sh; lock file exist [failed start attempt: $FAILNUM]"
	fi

	exit;
fi
echo $$ >/tmp/autoretrieve.lock
[ -f /tmp/autoretrieve.lockfailed ] && rm /tmp/autoretrieve.lockfailed


D1=`date '+%Y%m%d' --date="$HRS hours ago"`
T1=`date '+%H%M%S' --date="$HRS hours ago"`

if [ $HRS -gt 24 ]; then
	HRS2=`expr $HRS - 24`
	D0=`date '+%Y%m%d' --date="$HRS2 hours ago"`
else
	D0=`date '+%Y%m%d'`
fi


PID=$$
T=/tmp/q.$PID


echo ---------------
date

cat <<-ENDOFQDMP > /tmp/q2.dmp
	(0008,0052) CS [STUDY]
	(0020,000d) UI [] # Study IUID
	(0020,1208) IS [] # Number of Study related images
ENDOFQDMP
$DCMTK_HOME/bin/dump2dcm /tmp/q2.dmp /tmp/q2.dcm >/dev/null 2>&1

echo "(0008,0052) CS [STUDY]" > /tmp/s.dmp
$DCMTK_HOME/bin/dump2dcm /tmp/s.dmp /tmp/s.dcm >/dev/null 2>&1

# get all the study IUIDs and # of objects from the archive

echo "requesting recent studies and number of images from the PACS"

echo "1: $D1 $T1-"

$DCMTK_HOME/bin/findscu -$QRROOT --aetitle $AE_TITLE \
		--key 0008,0020="${D1}" --key 0008,0030="${T1}-" \
		--call $QRAE_TITLE $QRIP $QRPORT /tmp/q2.dcm > $T.res

if [ "$D0" != "$D1" ] ; then

	echo "2: $D0-"

	$DCMTK_HOME/bin/findscu -$QRROOT --aetitle $AE_TITLE \
      --key 0008,0020="${D0}-"  --key 0008,0030="000000-" \
      --call $QRAE_TITLE $QRIP $QRPORT /tmp/q2.dcm >> $T.res

fi

echo PACS data collected at `date`

cp $T.res /tmp/lastresult.res
#get STUDY IUIDs

awk '
$1=="(0020,000d)"  {s=$3;}
$1=="(0020,1208)"  {if (s ~ "^\\[.*\\]$" ) { s= s "\t" $3;
      gsub("\\[","",s); gsub("\\]","",s); print s; } }' $T.res | sort | awk '($2 != "0") {print $0}' > $T.ls


# get all study IUIDs and number of images in the system

${TOOLS_HOME}/sql.sh "select STYIUID,NUMOFIMG from $STUDYTBL;" -N | sort > $T.db.ls

# get difference; list of studies not in our database

diff $T.ls $T.db.ls | grep "^<" | sed -e "s/< //g" -e "s/\t.*$//g" > $T.mv


# retrieve all missing or partial studies

cat $T.mv | head -8 | while read styiuid number; do

	echo retrieving $styiuid from the PACS;

	$DCMTK_HOME/bin/movescu -$QRROOT --move $AE_TITLE --aetitle $AE_TITLE \
			--call $QRAE_TITLE $QRIP $QRPORT --key 0020,000d=$styiuid /tmp/s.dcm

#	break; # Get one study at a time; removing this break gets all missing ones

done


cp $T.ls /tmp/qr.ls # preserve for debugging
rm $T.*
rm /tmp/autoretrieve.lock
[ -f /tmp/autoretrieve.lockfailed ] && rm /tmp/autoretrieve.lockfailed
