#!/bin/bash
#
# Converts old storestate.rec files into new ones and creates reference counters for studies
#

. $HOME/etc/setenv.sh


deviceList=""
countOfDevices=0

declare -a devArray
declare -a ipArray

function fillDevices_()
{
	# devRes=`echo "select CONCAT(ID,'|',HOST) from $TARGETTBL where TYPE='C' OR TYPE='P';" | \
	devRes=`echo "select CONCAT(ID,'|',HOST) from $TARGETTBL;" | \
			$MYSQL_HOME/bin/mysql -N $DBNAME`
	if [ $? -ne 0 ] ; then
		echo "MySQL Error" >&2
		return 1
	fi

	for device in $devRes ; do
		id=`echo $device | cut -f 1 -d "|"`
		ip=`echo $device | cut -f 2 -d "|"`
		devArray[$countOfDevices]="$id"
		ipArray[$countOfDevices]="$ip"
		countOfDevices=$(( $countOfDevices + 1 ))
	done
	return 0
}

# $1 : IP address of [child] device
#
function getNameForIP_()
{
	i=0;
	while [ $i -lt $countOfDevices ] ; do
		if [ "${ipArray[$i]}" = "$1" ] ; then
			echo "${devArray[$i]}"
			return $i
		fi
		i=$(( $i + 1 ))
	done

	return $i
}

function getIPForName_()
{
	i=0;
	while [ $i -lt $countOfDevices ] ; do
		if [ "${devArray[$i]}" = "$1" ] ; then
			echo "${ipArray[$i]}"
			return $i
		fi
		i=$(( $i + 1 ))
	done

	return $i
}

# $1 : study instance UID
function undo_()
{
	# remove storestate.rec file
	$UTILS_HOME/bin/storestate --study "$1" -X

	# remove STUDYDIR reference counter
	rm -f $LOCK_VAR/reflock_STUDYDIR_$1
}


# $1 : study instance UID
#
function convertStorestate_()
{
	# convert old storestate.rec file into new one:
	# SenderIP  -> OwnerDevice
	# StagedIPs -> StagedDevices
	# Other fields are the same

	ProcessMode=""
	StageState=""
	Retrieved=""
	FromQR=""
	StudyOwner=""
	DontEdit=""
	DontDelete=""
	SenderIP=""
	StagedIPs=""
	. "`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $1`/storestate.rec"

	if [ $? -ne 0 ] ; then
		echo "ERROR $1 Could not include old storestate.rec file" >&2
		return 1
	fi

	storestateParams=""
	if [ -n "$SenderIP" ] ; then
		OwnerDevice=`getNameForIP_ $SenderIP`
		[ -n "$OwnerDevice" ] && storestateParams="$storestateParams --setowner \"$OwnerDevice\""
	fi

	if [ -n "$StagedIPs" ] ; then
		# unique staged list
		stagedList=`for i in $StagedIPs ; do echo $i; done | sort | sort -u`

		for staged in $stagedList ; do
			StagedDevice=`getNameForIP_ $staged`
			[ -n "$StagedDevice" ] && storestateParams="$storestateParams --setstaged \"$StagedDevice\""
		done
	fi

	[ -n "$ProcessMode" ] && storestateParams="$storestateParams --processmode \"$ProcessMode\""
	[ -n "$StageState" ] && storestateParams="$storestateParams --stagestate \"$StageState\""
	[ "$Retrieved" = "yes" -o "$FromQR" = "yes" ] && storestateParams="$storestateParams --retrieved=yes"
	[ "$StudyOwner" = "yes" ] && storestateParams="$storestateParams --owner=yes"
	[ "$DontEdit" = "yes" ] && storestateParams="$storestateParams --dontedit=yes"
	[ "$DontDelete" = "yes" ] && storestateParams="$storestateParams --dontdelete=yes"

	eval $UTILS_HOME/bin/storestate --study "$1" -C $storestateParams

	if [ $? -ne 0 ] ; then
		echo "ERROR $1 Could not convert storestate.rec file" >&2
		return 1
	fi

	return 0
}

# Create STUDYDIR reference counter
#
# $1 : study instance UID
#
function createSTUDYDIR_()
{
	OwnerDevice=""
	StagedDevices=""
	studydir=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $study`
	. $studydir/.info/storestate.rec
	if [ $? -ne 0 ] ; then
		echo "ERROR $1 Could not include storestate.rec file" >&2
		return 1
	fi

	# Add child server references
	if [ -n "$OwnerDevice$StagedDevices" ] ; then
		for i in $OwnerDevice $StagedDevices ; do
			$UTILS_HOME/bin/refcount -n "child_$i" -t "STUDYDIR" -i "$1" >>$LOG/refcount.error 2>&1
			if [ $? -ne 0 ] ; then
				echo "ERROR $1 could not add child_$i reference to STUDYDIR" >&2
				return 1
			fi
		done
	fi

	# Add file references
	for file in `ls $studydir/` ; do
		[ "$file" = "storestate.rec" ] && continue

		$UTILS_HOME/bin/refcount -n "$file" -t "STUDYDIR" -i "$1" >>$LOG/refcount.error 2>&1
		if [ $? -ne 0 ] ; then
			echo "ERROR $1 could not add $file reference to STUDYDIR" >&2
			return 1
		fi
	done

	return 0
}


CWD=`pwd`

fillDevices_
[ $? -ne 0 ] && exit 1

for study in `$REPOSITORYHANDLER_HOME/bin/listDirectories $REPOSITORY_DATA_DICOM` ; do
	studydir=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $study`
	if [ -f "$studydir/.info/storestate.rec" ] ; then
		echo "OK $study already converted" >&2
		continue
	fi

	if [ ! -f "$studydir/storestate.rec" ] ; then
		echo "ERROR $study Missing storestate.rec file" >&2
		continue
	fi

	convertStorestate_ $study
	[ $? -ne 0 ] && continue

	createSTUDYDIR_ $study
	if [ $? -ne 0 ] ; then
		undo_ $study
		continue
	fi

	st2=`echo "select STYIUID from $STUDYTBL where STYIUID='$study';" | $MYSQL_HOME/bin/mysql -N $DBNAME`
	if [ $? -ne 0 ] ; then
		echo "ERROR $study MySQL error" >&2
		undo_ $study
		exit 1
	fi

	if [ "$st2" = "$study" ] ; then
		# create STUDYDB reference counter
		$UTILS_HOME/bin/refcount -n "database" -t "STUDYDB" -i "$study" >>$LOG/refcount.error 2>&1
		if [ $? -ne 0 ] ; then
			echo "ERROR $study could not add database reference to STUDYDB" >&2
			undo_ $study
			continue
		fi

		fileList=`echo "select concat(FNAME, '#', STORELEVEL) from $OBJECTTBL where STYIUID='$study';" | $MYSQL_HOME/bin/mysql -N $DBNAME`
		if [ $? -ne 0 ] ; then
			echo "ERROR $study MySQL error" >&2
			undo_ $study
			exit 1
		fi

		for fileLevel in $fileList ; do
			file=`cut -f 1 -d "#" <<<"$fileLevel"`
			level=`cut -f 2 -d "#" <<<"$fileLevel"`
			# create STUDYDB reference counter
			$UTILS_HOME/bin/refcount -n "database" -t "OBJECTFILE" -i "${study}/${file}" >>$LOG/refcount.error 2>&1
			if [ $? -ne 0 ] ; then
				echo "ERROR $study could not add ${study}_${file} reference to OBJECTFILE" >&2
				undo_ $study
				continue
			fi

			if [ -n "$level" -a "$level" != "C" ] ; then
				# create legacy flag
				$UTILS_HOME/bin/refcount -n "storelevel" -t "OBJECTFILE" -i "${study}/${file}" >>$LOG/refcount.error 2>&1
			fi
		done
	fi

	rm -f $studydir/storestate.rec

	echo "OK $study"
done

exit 0
