#!/bin/bash
#
#
#

BaseName=$( basename $( readlink -f "$0" ) )
BaseDir=$( dirname $( readlink -f "$0" ) )

CrashInfoAwk="${BaseDir}/parseCrashInfo.awk"


if [ ! -f "${CrashInfoAwk}" ] ; then
	echo "Fatal error: '${CrashInfoAwk}' does not exist" >&2
	exit 1
fi


#
# Prints usage info
#
# [$@] - error message(s)
#
function printUsage_() {
	[ -n "$1" ] && echo "$@" >&2

	cat <<-EOF
		Parses viewerinfo logs and prints the list of crash logs in the log files, if any.
		Viewerinfo files can be either gzipped or plain text.

		Usage: ${BaseName} [-h] [-o <output>] <viewerinfo> ...

		    -h               : prints this help
		    -o <output>      : prints the list to this output file instead of standard output
		    <viewerinfo>     : viewerinfo log file(s)
	EOF

	[ -n "$1" ] && exit 1

	exit 0
}


#
# Lists the crash logs from stdin
#
function listCrashLogs_() {
	sed 's;\r$;;' | gawk -f "${CrashInfoAwk}"
}


#
# Processes a file
#
function process_() {
	local file="$1"

	gunzip -t "$file" >/dev/null 2>&1
	if [ $? -eq 0 ] ; then
		zcat "$file"
	else
		cat "$file"
	fi | listCrashLogs_	
}



OutputFile=""


while getopts 'ho:' OPT; do
        case $OPT in
                h) printUsage_ ;;
                o) OutputFile="$OPTARG" ;;
                *) printUsage_ "Invalid Option: $OPT" ;;
        esac
done

shift $(( $OPTIND - 1 ))

[ -z "$1" ] && printUsage_ "Missing viewerinfo file"


NumFiles="$#"

while [ -n "$1" ] ; do
	[ "$NumFiles" -gt 1 ] && echo "=============== $1 ===============" && echo
	if [ ! -f "$1" ] ; then
		echo "File '$1' does not exist" >&2
		shift
		continue
	fi

	process_ "$1"
	shift
done | if [ -n "$OutputFile" ] ; then cat - > "$OutputFile" ; else cat - ; fi
