#!/bin/bash
#
# --------------------------------------------------------------------------------------
# [igor]   Bogus language generator
#
# Creates a bogus default language (Messages.properties and Fields.properties) and sets
# English (en_US) as a regular language                   
# --------------------------------------------------------------------------------------
#
# Usage:
#
# command [-option]
#
# default (no arguments): 		Generates bogus language and sets it as the default language;
#								Notice that it is used when some resource is missing in the
#								selected language. This is not related with the server default
#								language that can be set via GUI!
# -u:							Undo option: sets Messages_en_US.properties and
#								Fields_en_US.properties back to the default files.
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

USAGE="\n Bogus language generator \n\n

Creates a bogus default language (Messages.properties and Fields.properties) and sets English\n (en_US) as a regular language.\n\n\n

Usage:	command [-option]\n\n

Options:\n\n

default (no arguments):\t Generates bogus language and sets it as the default language;\n \t\t\t\t Notice that it is used when some resource is missing in the selected\n\t\t\t\t language. This is not related with the server default language that\n\t\t\t\t can be set via GUI!\n\n
-u:\t\t\t\t						Undo option: sets Messages_en_US.properties and\n
\t\t\t\t  Fields_en_US.properties back to the default files.\n"

undo="false"
## The tester/developer can change up to this number of resources and the Messages.properties will
## be still considered a bogus.
TOLERANCE=40
BOGUS_ACRO=00
BOGUS_NAME=Bogus
BOGUS_DEF="$BOGUS_ACRO$BOGUS_ACRO, $BOGUS_NAME ($BOGUS_ACRO)"
BOGUS_MESSAGES=$CLASSES_HOME/imagemed/resources/Messages_00_00.properties
BOGUS_FIELDS=$CLASSES_HOME/imagemed/resources/Fields_00_00.properties

while test -n "$1"
do
	case "$1" in

		-h | --help	) 
			echo -e $USAGE
    		exit 0
		;;

	    -u | --undo)
        	undo="true"
        ;;

		*)
			echo "Invalid option!"
			exit 1		
		;;
	esac

	shift

done

echo

if [ "$undo" = "false" ]; then

	## Checking how many tags are not equal to its content.
	grep -Ev '^\s*$|^#|^\s*#' < $CLASSES_HOME/imagemed/resources/Messages.properties > $TMP/bogustest$$
	sed -i 's/\(.*\)=\1//' $TMP/bogustest$$
	regularResources=`grep -v '^$' $TMP/bogustest$$ | wc -l`

	## It is possible that the developer/tester edited few lines in the bogus, which remains a bogus
	## and shouldn't be copied. TOLERANCE should handle this case.
	if [ $regularResources -gt $TOLERANCE ]; then

		echo "WARNING: To avoid session inconsistencies, please log out any user using the server."
		echo "Press any key to continue or ESC to cancel..."
		read -s -n1 key
		case $key in $'\e') exit 0;; esac

		echo
		echo "Moving default English files to the new en_US files..."

		cp -f $CLASSES_HOME/imagemed/resources/Messages.properties $CLASSES_HOME/imagemed/resources/Messages_en_US.properties
		cp -f $CLASSES_HOME/imagemed/resources/Fields.properties $CLASSES_HOME/imagemed/resources/Fields_en_US.properties 

		echo 'Converting default resource files into bogus language...'

		sed -i 's/\(^[^#].*\)=.*/\1=\1/' $CLASSES_HOME/imagemed/resources/Messages.properties
		sed -i 's/\(^[^#].*\)=.*/\1=\1/' $CLASSES_HOME/imagemed/resources/Fields.properties

		## Making available the bogus language also as a regular language. This can be used to test
		## missing localizations keeping the GUI readable for English speakers.

		echo "Making available the bogus language as a regular language..."

		cp -f $CLASSES_HOME/imagemed/resources/Messages.properties $BOGUS_MESSAGES
		cp -f $CLASSES_HOME/imagemed/resources/Fields.properties $BOGUS_FIELDS

		## Double checking if the bogus language is not already listed and adding it.
		bogusExists=`grep "$BOGUS_DEF" < $CASES_HOME/cfg/languages.conf` 
		chmod +w $CASES_HOME/cfg/languages.conf
		[[ -z $bogusExists ]] && echo "$BOGUS_DEF" >> "$CASES_HOME/cfg/languages.conf" 
		chmod -w $CASES_HOME/cfg/languages.conf


		echo
		echo "Restart apache to load the new resource files!"

	else
		echo "The default file is already a bogus!"
	fi 

	rm -f $TMP/bogustest$$

else

	if [ -f $CLASSES_HOME/imagemed/resources/Fields_en_US.properties ] && [ -f $CLASSES_HOME/imagemed/resources/Messages_en_US.properties ]; then
		
		echo -e "WARNING: Please be sure that there is no user using the bogus language and that the server default\nlanguage is also not the bogus."
		echo "Press any key to continue or ESC to cancel..."
		read -s -n1 key
		case $key in $'\e') exit 0;; esac
		#TODO fix all these cases automatically

		echo
		echo "Rolling back English files to default..."

		cp -f $CLASSES_HOME/imagemed/resources/Messages_en_US.properties $CLASSES_HOME/imagemed/resources/Messages.properties 
		cp -f $CLASSES_HOME/imagemed/resources/Fields_en_US.properties $CLASSES_HOME/imagemed/resources/Fields.properties

		echo "Removing the bogus language option..." 
		sed -i "/$BOGUS_DEF/d" $CASES_HOME/cfg/languages.conf		

		rm -f $CLASSES_HOME/imagemed/resources/Messages_en_US.properties
		rm -f $CLASSES_HOME/imagemed/resources/Fields_en_US.properties

		rm -f $BOGUS_MESSAGES
		rm -f $BOGUS_FIELDS
	
		echo
		echo "Restart apache to load the new resource files!"
	
	else

		echo "ERROR: en_US files do not exist. English might be already the default."	

	fi

fi

echo