#!/bin/bash
#
# Component: tools
#
# Description:
#   Checks if all the DICOM objects listed in the listfile are
#   properly archived.
#
# Each line in listfile should look like:
#
#  <Study Instance UID> <Object file name>
#
# The script will check each object in the archive and will generate output line
# for each object:
#
# <check result> <Study Instance UID> <Object File Name>
#
# Where check result can be:
#
#  NOSTUDYARCHIVED  :   The study could not be found in the Archive database
#  NOOBJECTARCHIVED :   The study can be found in the Archive database,
#                   :   but the object is missing from the archive
#  NODATA           :   The object is in the archive but missing from $REPOSITORY_DICOM_DATA
#                   :   This case supposed to be OK.
#  DIFFER           :   The object is in both archive and $REPOSITORY_DICOM_DATA, but the two file differs
#  OK               :   The object is in both archive and $REPOSITORY_DICOM_DATA, and the two file is the same
#
# Usage:
#   isarchived.sh <listfile>
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

[ ! -f "$1" ] && exit 1

Archive="no"

[ -f "$ARCHIVE_VAR/archive.cfg" ] && . $ARCHIVE_VAR/archive.cfg

if [ "$Archive" != "yes" ] ; then
	$LOGGER -p $LOG_FACILITY.info -t "$TAG" "No archive has been set"
	echo "No archive has been set."
	exit 1
fi


Pid=$$
DATE=`date +%Y%m%d%H%M%S`
i=0

cat $1 | while read study file ; do
	studydir=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $study`
	
	archdir=`echo "select ARCHDIR from Archive where STYIUID='$study';" |\
			$MYSQL_HOME/bin/mysql -N $DBNAME 2>/dev/null`
	if [ $? -ne 0 ] ; then
		echo "MySQL error in select ARCHDIR" 1>&2
		exit 1
	fi

	if [ -z "$archdir" ] ; then
		# Not found in archive table

		echo "NOSTUDYARCHIVED $study $file"
	else
		if [ -f "$ARCHIVE_DIR/$archdir/$study/$file" ] ; then

			if [ -f "$studydir/$file" ] ; then
				cmp $studydir/$file $ARCHIVE_DIR/$archdir/$study/$file
				if [ $? -ne 0 ] ; then
					echo "DIFFER $study $file"
				else
					echo "OK $study $file"
				fi
			else
				echo "NODATA $study $file"
			fi
    	else
			echo "NOOBJECTARCHIVED $study $file"
		fi
	fi

done
